###SET WORKING DIRECTORY FIRST!!!!!
#upload packages
#three functions to upload SIT package
con = gzcon(url('http://www.systematicportfolio.com/sit.gz', 'rb'))
source(con)
close(con)
load.packages('quantmod')

library(igraph)
library(SIT)
library(quantmod)

#correlation function
clean.cor = function(ret, threshold = 0.5) {
  cor_mat = cor(coredata(ret), use='complete.obs',method='pearson')
  cor_mat[ abs(cor_mat) < threshold] = 0
  keep.index = rowSums(cor_mat != 0) > 1
  cor_mat = cor_mat[keep.index, keep.index]
  cor_mat[ lower.tri(cor_mat, diag=TRUE) ] = 0
  cor_mat
}

#graph function
plot.cor = function(ret, threshold = 0.5) {
  cor_mat = clean.cor(ret, threshold)
  
  transform.fn = function(x) x
  transform.fn = function(x) sqrt(2.0 * ( 1 - x ) )
  transform.fn = function(x) 1 - abs( x )
  
  dist = transform.fn( cor_mat )
  dist[cor_mat == 0] = 0
  
  graph = graph.adjacency(dist, weighted=TRUE, mode='upper')
  mst = minimum.spanning.tree(graph)
  
  breaks = c(0,55,60,65,70,100)
  cols = spl('black,blue,red,green,orange')
  labels = 1:5
  
  factor = cut(E(mst)$weight, breaks = transform.fn(breaks/100), labels = labels)
  for(i in labels)
    E(mst)[ factor == i ]$color = col.add.alpha(cols[i],150)
  
  set.seed(100)
  par(mar=c(1,1,1,1))
  plot(mst,vertex.size=5, vertex.color=NA, vertex.frame.color=NA, edge.width = 3) 
  
  legend('bottomleft', title='Minimum Spanning Tree', cex=0.75, pch=16, bty='n', ncol=2,
         col=spl('black,blue,red,green,orange'), 
         legend=spl('>70%,65-70,60-65,55-60,50-55')
  )	
  
  # full graph
  factor = cut(E(graph)$weight, breaks = transform.fn(breaks/100), labels = labels)
  for(i in labels)
    E(graph)[ factor == i ]$color = col.add.alpha(cols[i],100)
  
  set.seed(100)
  plot(graph,vertex.size=5, vertex.color=NA, vertex.frame.color=NA, edge.width = 3) 
  
  legend('bottomleft', title='Full Graph', cex=0.75, pch=16, bty='n', ncol=2,
         col=spl('black,blue,red,green,orange'), 
         legend=spl('>70%,65-70,60-65,55-60,50-55')
  )	
}

#upload data
data <- read.csv("Data.csv", header = TRUE, sep = ";")

#set data as time series
edhec_timeseries <- ts(data[,-1], start = c(1999, 12), frequency=12)
data <- as.xts(edhec_timeseries)

#calculate returns
ret = diff(log(data)) #all data

#set number of observation - as number of observations from the end of time series
# use function first(ret,...) to set observations from the beggining of time series
ret = first(ret, 85) #pre-crisis period
ret = ret[86:(nrow(ret)-85),] #crisis period
ret = last(ret, 85) #post-crisis period

#make MST graph
plot.cor(ret, 0.3)


#Tento zdrojov kd je upraven verze kdu ze Systematic Investor's Blog [online], 2015 [cit. 17.4.2017]. Dostupn z: http://systematicinvestor.github.io/Minimum-Spanning-Tree  